#ifndef __INTERFAC_H
#define __INTERFAC_H

#include <errno.h>

#ifdef __GNUC__			/* if compiled under gnu c  */
#define INLINE  inline          /* turn `inline' keyword on */
#else
#define INLINE			/* otherwise ignore it */
#endif

/*
 * implicit parameters
 */
#define IMPLICIT_LEFT 4
#define IMPLICIT_RIGHT 76
#define IMPLICIT_PARINDENT 5
#define IMPLICIT_HANGINDENT 1

/* 
 * exit codes
 */
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1

/* 
 * error messages 
 */
#define ERRNOMEM "not enough memory...\n"
#define ERRNOPOS "no good-looking output possible...\n"
#define ERRTOOLARGE "paragraph too large...\n"
#define ERRNONUM "cmdline parameters out of range...\n"
#define ERRNOFIL "invalid path or filename...\n"
#define ERRUNKNOWN "unknown error while parsing options\n"
#define ERRUSAGE "invalid command line\n\
Try `%s --help' for more information\n"
#define ERRRANGE "parameters out of range\n\
Try man pages for more information\n"

/* 
 * warning messages
 */
#define WINVALIDPAR "%s: invalid parameter `%s'\n"
#define WUNKNOWNOPT "%s: unknown option `%s'\n"
#define WMISSINGPAR "%s: missing parameter\n"

/*
 * other messages
 */
#define USAGE "usage\n\
-l, --left=NUMBER		left margin (default 4) \n\
-r, --right=NUMBER		right margin (default 76)\n\
-p, --parindent=NUMBER		paragraph indentation (default 5) \n\
-g, --hangindent=NUMBER		number of lines, \n\
				  which will be indented (default 1) \n\
-w, --words=DICTIONARY_FILE	dictionary file of unbreakable words \n\
-v, --version			outputs version information \n\
-h, --help			displays this help and exits \n\n\
BUGS report to: toman@horac.ta.jcu.cz\n"
#define VERSION "par, version 1.1 (by Kamil Toman)\n"

/*
 * arguments
 */
#define NOARG 0
#define REQUIREDARG 1
#define OPTIONALARG 2

extern unsigned char *progname, *inputrc;

/*
 * parsed parameters
 */
#define NUMOFPARAMS 4
#define LEFT param[0]
#define RIGHT param[1]
#define PARINDENT param[2]
#define HANGINDENT param[3]

/*
 * misc routines
 */
void parse_cmdline_parameters(int argc, char **argv, int param[NUMOFPARAMS]);
void err_exit(int code, const char *messg);

#define ALLOC(ptr, size); { if (( (ptr) = malloc((size))) == NULL)\
				err_exit(EXIT_FAILURE, ERRNOMEM); }
#define REALLOC(ptr, size); { if (( (ptr) = realloc(ptr, (size))) == NULL)\
				err_exit(EXIT_FAILURE, ERRNOMEM); }
#define ASSIGN_PAR(par, value)  {\
                                int num;\
                                num = strtol((value), NULL, 0);\
                                if (errno == ERANGE) {\
                                        fprintf(stderr, "%s: %s", progname, ERRRANGE);\
					err = 1;\
				}\
                                else\
                                        par = num; } 

#endif
